
from t_core.composer import Composer
from t_core.selector import Selector


class Layer(Composer):
    '''
        Applies the rules non-deterministically until none match.
    '''
    def __init__(self, rules):
        '''
            Applies the rules until none match.
            @param rules: The rules to apply.
        '''
        super(Layer, self).__init__()
        self.rules = rules
        self.S = Selector()
    
    def packet_in(self, packet):
        self.exception = None
        self.is_success = False
        while True:
            # Apply rule
            for rule in self.rules:
                packet = rule.packet_in(packet)
                if rule.is_success:
                    self.S.success_in(packet)
                    self.is_success = True
                else:
                    if rule.exception is not None:
                        self.exception = rule.exception
                        return packet
                    self.S.fail_in(packet)
            # Select
            packet = self.S.select()
            if not self.S.is_success:
                return packet
        return packet
